#############################################################################
#
# File:   MakeLibrary.mk
# Author: Dave Hefner
# Date:   Nov. 15, 2012
#
# This file is used to build test libraries.
#
# Copyright (C) 2012 Hewlett-Packard Co.  All rights reserved.
#
#############################################################################

include $(WC_ROOT)/common.mk

LIBRARY1:=$(subst /, ,$(abspath $(THIS_DIR)))
LIBRARY:=$(lastword $(LIBRARY1))

make-directories:=$(shell mkdir -p obj bin)

REQUIRED_DIRS:=$(LIB_OBJ_DIR) $(LIB_DIR)

.SECONDEXPANSION:
.SUFFIXES:

LIB_ROOT := $(THIS_DIR)
LIB_SRC := $(LIB_ROOT)src
LIB_INC := $(LIB_ROOT)inc
LIB_DIR := $(LIB_ROOT)bin
include $(LIB_SRC)/SOURCES
ifneq "$(strip $(wildcard $(PRODUCT_LIB)))" ""
    NEW_CPP_LIB_SOURCES := $(filter-out CpqCiDummy%.cpp,$(LIB_CPP_SOURCES))
    LIB_CPP_SOURCES=$(NEW_CPP_LIB_SOURCES)
endif
LIB_CPP_OBJECTS := $(notdir $(LIB_CPP_SOURCES:.cpp=.o))
LIB_C_OBJECTS := $(notdir $(LIB_C_SOURCES:.c=.o))
LIB_OBJ_DIR := $(abspath $(LIB_ROOT)obj)
LIB_OBJ_PREFIX := $(abspath $(LIB_ROOT)obj)
LIB_CPP_OBJECTS_PATH := $(addprefix $(LIB_OBJ_PREFIX)/,$(LIB_CPP_OBJECTS))
LIB_C_OBJECTS_PATH := $(addprefix $(LIB_OBJ_PREFIX)/,$(LIB_C_OBJECTS))
L1_LIBRARY := $(abspath $(LIB_DIR)/lib$(LIBRARY).so)
L1_STATIC_LIBRARY := $(abspath $(LIB_DIR)/lib$(LIBRARY).a)

library: $(L1_LIBRARY) $(L1_STATIC_LIBRARY)
	@touch $@

$(LIB_CPP_OBJECTS_PATH): $(ADDITIONAL_CPP_DEPENDENCIES)
$(LIB_CPP_OBJECTS_PATH): $(LIB_OBJ_DIR)/%.o: $(LIB_SRC)/$$*.cpp 
	${CXX} $(CXXFLAGS) $(CPPFLAGS) $(TARGET_ARCH) -I./inc $< -o $@

$(LIB_C_OBJECTS_PATH): $(LIB_OBJ_DIR)/%.o: $(LIB_SRC)/$$*.c 
	${CC} $(CFLAGS) $(CPPFLAGS) $(TARGET_ARCH) -I./inc $< -o $@

ifneq "$(MAKECMDGOALS)" "clean"
    -include $(LIB_OBJ_DIR)/*.d
endif

$(L1_LIBRARY): $(LIB_CPP_OBJECTS_PATH) $(LIB_C_OBJECTS_PATH) | $(REQUIRED_DIRS)
	${CXX} $(CXXFLAGS) -Wl,--warn-constructors $(LDFLAGS) $(CC_LDFLAGS_RPATH) -shared -Wl,-soname,$@ $^ -rdynamic -ldl -o $@.1.0
	@ln -sf $(L1_LIBRARY).1.0 $(L1_LIBRARY)
	@ln -sf $(L1_LIBRARY).1.0 $(L1_LIBRARY).1
	-@$(MAKE) -s -k --no-print-directory -C $(LIB_ROOT)unitTests clean

$(L1_STATIC_LIBRARY): $(LIB_CPP_OBJECTS_PATH) $(LIB_C_OBJECTS_PATH) | $(REQUIRED_DIRS)
	$(AR) rcs $@ $^
	-@$(MAKE) -s -k --no-print-directory -C $(LIB_ROOT)unitTests clean

.PHONY: check
check: $(L1_LIBRARY)
	-@$(MAKE) -k --no-print-directory -C $(LIB_ROOT)unitTests
	-@$(MAKE) -k --no-print-directory -C $(LIB_ROOT)unitTests check

.PHONY: indent
indent:
	../sbin/prettyCode.sh $(LIB_SRC)/*.cpp $(LIB_INC)/*.hpp $(LIB_INC)/*.h 

.PHONY: clean
clean:
	@rm -rf obj bin $(ADDITIONAL_CLEAN) library 
	-@$(MAKE) -s -k --no-print-directory -C $(LIB_ROOT)unitTests clean

