#
# Compute platform-specific variables.
#
HARDWARE_PLATFORM:=$(strip $(shell uname -i))

L1_VERSION_STRING:="$(shell svnversion)"

OS:=$(shell uname)

OS_VER:=$(shell uname -r)
OS_VERSION:=
ifeq "$(OS_VERSION)" ""
    OS_VERSION:=$(if $(findstring .el5,$(OS_VER)),RHEL5,)
endif
ifeq "$(OS_VERSION)" ""
    OS_VERSION:=$(if $(findstring .el6,$(OS_VER)),RHEL6_CENTOS6,)
endif
ifeq "$(OS_VERSION)" ""
    OS_VERSION:=$(if $(findstring .el7,$(OS_VER)),RHEL7_CENTOS7,)
endif
ifeq "$(OS_VERSION)" ""
    OS_VERSION:=$(if $(findstring generic,$(OS_VER)),GENERIC,)
endif
ifeq "$(OS_VERSION)" ""
    OS_VERSION:=$(if $(findstring mustang,$(OS_VER)),MUSTANG,)
endif
ifeq "$(OS_VERSION)" ""
    OS_VERSION:=$(if $(findstring balvenie,$(OS_VER)),MUSTANG,)
endif

ifeq "$(OS_VERSION)" "RHEL6_CENTOS6"
    XXRELEASE:=$(shell cat /etc/redhat-release)
    OS_VERSION:=$(if $(findstring CentOS,$(XXRELEASE)),CENTOS6,RHEL6)
    $(warn $(OS_VERSION))
endif
OS_VERSION:=GENERIC
ifeq "$(OS_VERSION)" "RHEL7_CENTOS7"
    XXRELEASE:=$(shell cat /etc/redhat-release)
    OS_VERSION:=$(if $(findstring CentOS,$(XXRELEASE)),CENTOS7,RHEL7)
    $(warn $(OS_VERSION))
endif

# In case anyone is using Debian, which doesn't seem to put any identifier
# in its kernel versioning.
ifeq "$(OS_VERSION)" ""
   OS_VER_ALT:=$(shell uname -a)
   OS_VERSION:=$(if $(findstring Debian,$(OS_VER_ALT)),DEBIAN)
endif

$(if $(OS_VERSION),,$(error OS version is not supported!))

include $(WC_ROOT)/TChain.mk

BUILD_ROOT:=$(realpath $(dir $(word $(words $(MAKEFILE_LIST)),$(MAKEFILE_LIST))))
BUILD_TOOLS_DROP:=$(realpath /vduBuildTools)

HAS_PTHREAD_SETNAME_NP:=$(if $(shell grep pthread_setname_np /usr/include/pthread.h),1,0)

.SUFFIXES:

SED 		= /bin/sed
MV		= /bin/mv

DEV_TYPE:=$(shell lsmod | grep -o hpsa -m 1 )

OS_NAME:=LINUX

BUILD_TOOLS_BIN:=${BUILD_TOOLS_DROP}/bin
BUILD_TOOLS_INC:=${BUILD_TOOLS_DROP}/include
BUILD_TOOLS_SRC:=${BUILD_TOOLS_ROOT}/src
BUILD_TOOLS_LIB:=${BUILD_TOOLS_DROP}/lib
BUILD_TOOLS_LIB64:=${BUILD_TOOLS_DROP}/lib64
BUILD_TOOLS_SHARE:=${BUILD_TOOLS_DROP}/share
BUILD_TOOLS_PKGCONFIG:=${BUILD_TOOLS_LIB}/pkgconfig:$(BUILD_TOOLS_LIB64)/pkgconfig

GSOAP_IMPORT:=${BUILD_TOOLS_SHARE}/gsoap/import

UNIT_TEST_REPORT_DIR:=$(BUILD_ROOT)/unitTestReports/

NEW_PATH:=$(BUILD_TOOLS_BIN):$(PATH)
PATH:=$(NEW_PATH)

BOOST_ROOT:=$(abspath $(BUILD_TOOLS_DROP)/Boost/boost_1_71_0)
BOOST_INC:=$(BOOST_ROOT)/include
BOOST_LIB:=$(BOOST_ROOT)/lib
BOOST_LIBRARIES:=-lboost_regex -lboost_system -lboost_thread -lboost_filesystem -lboost_log_setup -lboost_log -lboost_date_time -lboost_serialization -lboost_stacktrace_addr2line
-include ./BOOST_OVERRIDE

INC_DIRS+=-I$(WC_ROOT) -I$(BOOST_INC) -I$(BUILD_TOOLS_INC)

#Create Tag file
CTAGS:=ctags -f TAGS ./src/*.cpp ./lib/src/*.cpp ./lib/inc/*.h
#$(shell ${CTAGS})

#
# Compute the PKG_CONFIG_PATH variable.
#
PKG_CONFIG_BINARY:=$(wildcard $(TOOL_CHAIN_BIN)/pkg-config)
$(if $(PKG_CONFIG_BINARY),,$(error pkg-config was not found!))
PKG_CONFIG_PATH:=$(word 1,$(dir $(shell find /usr -name glib-2.0.pc)))
ifeq "$(HARDWARE_PLATFORM)" "aarch64"
    NEW_PKG_CONFIG_PATH:=$(BUILD_TOOLS_PKGCONFIG):$(PKG_CONFIG_PATH):$(WC_ROOT)/SharedFiles
    TARGET_ARCH:=
else
    NEW_PKG_CONFIG_PATH:=$(BUILD_TOOLS_PKG_CONFIG):$(TOOL_CHAIN_PKG_CONFIG):$(TOOL_CHAIN_PKG_CONFIG64):$(PKG_CONFIG_PATH):$(WC_ROOT)/SharedFiles
endif
export PKG_CONFIG_PATH:=$(NEW_PKG_CONFIG_PATH)

ifneq "$(MAKECMDGOALS)" "clean"

    PRODUCT_LIB:=$(strip $(wildcard /usr/lib64/libcpqci64.so))

    MH_LIB:=$(abspath $(BUILD_ROOT)/mhlib/bin/libmhlib.so)
    VDU_LIB:=$(abspath $(BUILD_ROOT)/lib/bin/liblib.so)
    GEN_LIB:=$(abspath $(BUILD_ROOT)/genlib/bin/libgenlib.so)
    rSCSIClient_LIB:=$(abspath $(BUILD_ROOT)/rSCSIClientLib/lib/librSCSIClient.a)
    DP_LIB:=$(abspath $(BUILD_ROOT)/dplib/bin/libdplib.so)
    RCIM_LIB:=$(abspath $(BUILD_ROOT)/DroppedComponents/Drop/lib/librcim.a)
    GMOCK_MAIN:=$(abspath $(BUILD_TOOLS_LIB)/gmock_main.a)
    SOCKETS_LIB:=$(abspath $(BUILD_TOOLS_LIB)/libSockets.a)
    LOG4CPLUS_LIB:=$(abspath $(BUILD_TOOLS_LIB)/liblog4cplus.a)
    LIBRCIM:=$(abspath $(BUILD_ROOT)/DroppedComponents/Drop/lib/librcim.a)
    LIBRCIM_SERVICE:=$(abspath $(BUILD_ROOT)/RCIMService/bin/libRCIMService.a)
    LIBRCIMCLI:=$(abspath $(BUILD_ROOT)/DroppedComponents/Drop/lib/librcimcli.a)

ifeq "$(STANDALONE)" "true"
    L1_LIBS:=$(GEN_LIB) $(LOG4CPLUS_LIB) $(TBB_LIBRARIES)
else
    L1_LIBS:=$(MH_LIB) $(VDU_LIB) $(GEN_LIB) $(DP_LIB) $(LIBRCIM) $(LIBRCIMCLI) $(LIBRCIM_SERVICE)
endif

    HAS_UDEVADM:=$(strip $(if $(strip $(shell which udevadm 2>/dev/null)),1,0))

    rSCSIClient=$(if $(findstring true,$(STANDALONE)),,rSCSIClient)

    PKG_CFLAGS:=`$(PKG_CONFIG_BINARY) --print-errors --cflags zlib libssh2 openssl libcurl python xerces-c libusb-1.0 log4cplus json-c sqlite3 glib-2.0 liburcu-qsbr $(rSCSIClient) libzmq protobuf libcares`
ifneq "$(STANDALONE)" "true"
    PKG_CFLAGS+=`Sockets-config`
endif
    PKG_LIBFLAGS:=`$(PKG_CONFIG_BINARY) --print-errors --libs zlib libssh2 openssl libcurl python xerces-c libusb-1.0 json-c sqlite3 glib-2.0 liburcu-qsbr $(rSCSIClient) libzmq protobuf libcares`
ifneq "$(STANDALONE)" "true"
    PKG_LIBFLAGS+=`Sockets-config`
endif

    LDFLAGS_RPATH:=$(TOOL_CHAIN_RPATH) $(BUILD_TOOLS_RPATH) -Wl,-rpath=$(BUILD_TOOLS_LIB)/expect5.45.4 -Wl,-rpath=$(BOOST_LIB) -Wl,-rpath=$(TBB_LIBRARIES_ROOT)
    LDFLAGS_RPATH+=-Wl,-rpath-link=$(BUILD_TOOLS_LIB)/expect5.45.4 -Wl,-rpath-link=$(BOOST_LIB) -Wl,-rpath-link=$(TBB_LIBRARIES_ROOT)
    LDFLAGS:=-L$(BUILD_TOOLS_LIB) -L$(BUILD_TOOLS_LIB64) -L$(BUILD_TOOLS_LIB)/expect5.45.4 -L$(BOOST_LIB) -L$(TBB_LIBRARIES_ROOT)
    LDFLAGS+=$(LDFLAGS_RPATH)

ifeq "$(STANDALONE)" "true"
    LIBS:=$(L1_LIBS) $(BOOST_LIBRARIES) $(TBB_LIBRARIES) -lexpect5.45.4 -ltcl8.5 -lsgutils2 $(PKG_LIBFLAGS)
else
    LIBS:=$(L1_LIBS) $(PRODUCT_LIB) $(RCIM_LIB) $(BOOST_LIBRARIES) $(TBB_LIBRARIES) $(SOCKETS_LIB) $(LOG4CPLUS_LIB) -lexpect5.45.4 -ltcl8.5 -lsgutils2 $(PKG_LIBFLAGS) $(TBB_LIBRARIES)
endif
    LIBS+=-lrt -lutil -ldl -lc -lpthread

    CPPFLAGS+=-DL1_VERSION_STRING=$(L1_VERSION_STRING) -DBOOST_ALL_DYN_LINK -DBOOST_STACKTRACE_LINK -DBOOST_STACKTRACE_USE_ADDR2LINE -DBOOST_STACKTRACE_ADDR2LINE_LOCATION=$(TOOL_CHAIN_BIN) -rdynamic -pthread -DHAS_PTHREAD_SETNAME_NP=$(HAS_PTHREAD_SETNAME_NP)
ifeq "$(STANDALONE)" "true"
    CPPFLAGS+=-include $(BUILD_ROOT)/genlib/inc/declareTemplates.hpp -D_LINUX -MMD -c $(INC_DIRS) $(PKG_CFLAGS)
else
    CPPFLAGS+=-include $(BUILD_ROOT)/genlib/inc/declareTemplates.hpp -D_LINUX -MMD -c $(INC_DIRS) -DHAS_UDEVADM=$(HAS_UDEVADM) $(PKG_CFLAGS) -I$(BUILD_ROOT)/DroppedComponents/Drop/include -DWITH_SOCKET_CLOSE_ON_EXIT
endif
    CFLAGS+=-Wno-unused -Wno-unused-parameter -Wno-error=deprecated-declarations -rdynamic

endif

-include ./Overrides.mk

export

